# -*- coding: utf-8 -*-
#
# Graphical interface for the eSpeak speech synthesizer
#
# Copyright © 2009-2012 Siegfried-Angel Gevatter Pujals <siegfried@gevatter.com>
# Copyright © 2009 Joe Burmeister <joe.a.burmeister@googlemail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

import os
import gtk
import gobject
import gtksourceview2
import gettext
import locale
import gtkspell
from optparse import OptionParser
from espeak import espeak

from resources import *
import about_dialog
from language import LanguageManager
from voicebox import VoiceBox

gtk.gdk.threads_init()

class Application:

    _espeak_language = None
    _language_manager = None
    _voicebox = None
    _spell = None
    
    @property
    def _espeak_gender(self):
        gender = settings.get_string(settings_gender)
        return espeak.Gender.Male if gender == 'male' else (
            espeak.Gender.Female if gender == 'female' else None)
    
    def __init__(self):
        self._builder = gtk.Builder()
        self._builder.set_translation_domain('espeak-gui')
        self._builder.add_from_file(interface_file)
        self._builder.connect_signals(self)
        
        self._recentmanager = gtk.RecentManager()
        
        for gender in ('male', 'female', 'any'):
            menuitem = self._builder.get_object('voice_gender_%s' % gender)
            menuitem.connect('toggled', self._e_voice_gender_toggled, gender)
            if gender == (settings.get_string(settings_gender) or 'any'):
                menuitem.set_active(True)
        
        selected_punctuation = settings.get_string(settings_punctuation) or 'any'
        for punctuation in ('all', 'custom', 'any'):
            menuitem = self._builder.get_object('punctuation_%s' % punctuation)
            menuitem.connect('toggled', self._e_punctuation_toggled, punctuation)
            if punctuation == selected_punctuation:
                menuitem.set_active(True)
        espeak.set_parameter(espeak.Parameter.Punctuation,
            getattr(espeak.Punctuation, selected_punctuation.capitalize()))
        
        self._main = self._builder.get_object('main_window')
        self._main.set_icon_from_file(icon_file)
        
        self._scrollbox = self._builder.get_object('scrolledwindow')
        self._playbutton = self._builder.get_object('button_read')
        self._pausebutton = self._builder.get_object('button_pause')
        self._speedspin = self._builder.get_object('speedspin')
        self._about_dialog = about_dialog.create_about_dialog(self._main)
        
        self._buffer = gtksourceview2.Buffer()
        self._buffer.connect('changed', self._e_text_changed)
        self._inputbox = gtksourceview2.View(self._buffer)
        self._inputbox.set_pixels_above_lines(2)
        self._inputbox.set_pixels_below_lines(2)
        self._inputbox.set_left_margin(4)
        self._inputbox.set_right_margin(4)
        self._inputbox.set_wrap_mode(gtk.WRAP_WORD);
        self._scrollbox.add(self._inputbox)
        #self._inputbox.connect('populate-popup', self._e_on_context_menu)
                
        self._language_manager = LanguageManager(self._get_voice_list())
        self._voicebox = VoiceBox(
            self._builder.get_object('voicebox'), self._language_manager)
        self._voicebox.set_voice(
            settings.get_string(settings_selected_voice) or \
            self._language_manager.get_default_language())
        
        self._playing = False
        self._paused = False
        self._speedspin.set_value(settings.get_int(settings_speed) or 160)
        
        espeak.set_SynthCallback(self._e_eventCB)
        
        self._speaking_tag = self._buffer.create_tag('speaking')
        self._speaking_tag.set_property('background', 'black')
        self._speaking_tag.set_property('foreground', 'white')
        
        self._speaking_text_offset = 0
        self._spoken_chars_offset = 0
        
        if arguments:
            self._load_files(*arguments)
        
        self._main.show_all()
        self._pausebutton.set_visible(False)
        gtk.main()
    
    def _autodetect(self):
        text = self._buffer.get_text(self._buffer.get_start_iter(),
            self._buffer.get_end_iter())
        return self._language_manager.autodetect(text)
    
    def synth(self, text):
        parameters = {}
        if self._espeak_gender:
            parameters['gender'] = self._espeak_gender
        if self._espeak_language not in (None, 'autodetect'):
            parameters['language'] = self._espeak_language
        else:
            guess = self._autodetect()
            self._voicebox.set_guess(guess)
            parameters['language'] = guess.get_language()
        print parameters
        espeak.set_voice(**parameters)
        return espeak.synth(text)
    
    def _get_voice_list(self):
        """ Returns the output of espeak.list_voices() after sorting it. """
        return sorted(espeak.list_voices(), cmp=locale.strcoll,
            key=lambda voice: voice.name)
    
    def _e_voicebox_changed(self, widget):
        (voice_name, voice_id) = self._voicebox.get_voice()
        self._active_voice = voice_name
        self._espeak_language = voice_id
        settings.set_string(settings_selected_voice, voice_id)
        if not self._spell:
            self._spell = gtkspell.Spell(self._inputbox)
        try:
            self._spell.set_language(voice_id.split('/', 1)[-1])
        except RuntimeError:
           # unsupported language
            self._spell.detach()
            self._spell = None
    
    def _set_voice(self, voice):
        self._voicebox.set_voice(voice)
    
    def _cleanup_speech_end(self):
        self._playing = False
        self._paused = False
        self._buffer.remove_tag(self._speaking_tag,
            self._buffer.get_start_iter(), self._buffer.get_end_iter())
        self._inputbox.show_all()
        self._inputbox.set_editable(True)
        self._playbutton.set_label(_('Play'))
        self._pausebutton.set_label(_('Pause'))
        self._pausebutton.set_visible(False)
        self._playbutton.show()
    
    def _e_eventCB(self, event, pos, length):
        
        gtk.gdk.threads_enter()
        
        if event == espeak.core.event_WORD:
            pos += (self._speaking_text_offset - 1)
            self._buffer.remove_tag(self._speaking_tag, self._buffer.get_start_iter(),
                self._buffer.get_end_iter())
            
            self._spoken_chars_offset = pos
            s = self._buffer.get_iter_at_offset(pos)
            e = self._buffer.get_iter_at_offset(length+pos)
            pos = self._inputbox.get_iter_location(s)
            
            adj = self._scrollbox.get_vadjustment()
            halfPage = adj.page_size / 2
            if (pos.y + pos.height) > (adj.value + halfPage):
                adj.set_value(min(adj.value+halfPage/2, adj.upper))
            elif pos.y < adj.value:
                adj.set_value(pos.y)
            
            self._buffer.apply_tag(self._speaking_tag, s, e)
            self._inputbox.show_all()
        
        if event == espeak.event_MSG_TERMINATED:
            self._cleanup_speech_end()
        
        gtk.gdk.threads_leave()
        
        return True
    
    def _play(self):
        if self._playing:
            if not self._paused:
                return
        else:
            self._playing = True
            self._spoken_chars_offset = 0
        
        self._inputbox.set_editable(False)
        text = ''
        
        if self._paused:
            self._speaking_text_offset = self._spoken_chars_offset
            text = self._buffer.get_text(
                self._buffer.get_iter_at_offset(self._spoken_chars_offset),
                self._buffer.get_end_iter())
        
        elif self._buffer.get_has_selection():
            selection = self._buffer.get_selection_bounds()
            
            s = selection[0]
            e = selection[1]
            
            self._speaking_text_offset = s.get_offset()
            text = self._buffer.get_text(s, e)
            self._buffer.select_range(self._buffer.get_start_iter(),
                self._buffer.get_start_iter())
        
        else:
            self._speaking_text_offset = 0
            text = self._buffer.get_text(self._buffer.get_start_iter(),
                self._buffer.get_end_iter())
        
        self._playbutton.set_label(_('Stop'))
        self._pausebutton.set_visible(True)
        
        if not self.synth(text):
            self._cleanup_speech_end()
    
    def _e_text_changed(self, *discard):
        text = self._buffer.get_text(self._buffer.get_start_iter(),
            self._buffer.get_end_iter())
        self._playbutton.set_sensitive(text != "")
    
    def _e_button_read_cd(self, *discard):
        if not self._playing:
            self._play()
        else:
            espeak.cancel()
            self._cleanup_speech_end()
        
        self._playbutton.show()
    
    def _e_button_pause_cd(self, *discard):
        if self._paused:
            self._play()
            self._pausebutton.set_label(_('Pause'))
            self._paused = False
        else:
            espeak.cancel()
            self._pausebutton.set_label(_('Play'))
            self._paused = True
    
    def _e_play(self, *discard):
        self._play()
        self._paused = False
    
    def _e_speed_changed(self, widget):
        speed = int(widget.get_value())
        espeak.set_parameter(espeak.Parameter.Rate, speed)
        settings.set_int(settings_speed, speed)
    
    def _e_new(self, *discard):
        self._buffer.delete(self._buffer.get_start_iter(),
            self._buffer.get_end_iter())
    
    def _e_new_window(self, *discard):
        new_instance()
    
    @staticmethod
    def _ensure_uri(filename):
        if filename.startswith('/'):
            return 'file://%s' % filename
        return filename

    @staticmethod
    def _ensure_filename(filename):
        if filename.startswith('file://'):
            return filename[7:]
        return filename
    
    def _load_files(self, *filenames):
        text = ''
        multiple_files = len(filenames) > 1
        for filename in filenames:
            filename = self._ensure_filename(filename)
            if multiple_files:
                if text:
                    text += '\n' * (3 - len(text[-2:]) + len(text[-2:].strip()))
                text += _('File:') + ' %s\n\n' % os.path.basename(filename)
            try:
                f = open(filename)
            except IOError, error:
                text += _('This file couldn\'t be opened: %(strerror)s.') % \
                    { 'strerror': error.strerror }
            else:
                content = f.read()
                if content:
                    text += content
                elif multiple_files:
                    text += _('This file is empty.')
                del content
                f.close()
                self._recentmanager.add_item(
                    self._ensure_uri(os.path.abspath(filename)))
        self._buffer.set_text(text)
    
    @staticmethod
    def _set_file_filters(dialog):
        filter = gtk.FileFilter()
        filter.set_name(_('All files'))
        filter.add_pattern('*')
        dialog.add_filter(filter)
        
        filter = gtk.FileFilter()
        filter.set_name(_('Plain text'))
        filter.add_mime_type('text/plain')
        filter.add_pattern('*.txt')
        dialog.add_filter(filter)
    
    def _e_open(self, *discard):
        dialog = gtk.FileChooserDialog(_('Open...'), self._main,
            gtk.FILE_CHOOSER_ACTION_OPEN, (gtk.STOCK_CANCEL,
            gtk.RESPONSE_CANCEL, gtk.STOCK_OPEN, gtk.RESPONSE_OK))
        dialog.set_select_multiple(True)
        dialog.set_default_response(gtk.RESPONSE_OK)
        self._set_file_filters(dialog)
        
        response = dialog.run()
        if response == gtk.RESPONSE_OK:
            self._load_files(*dialog.get_filenames())
        
        dialog.destroy()
    
    def _e_save(self, *discard):
        # TRANSLATORS: This is the title of the file selection dialog
        dialog = gtk.FileChooserDialog(_('Save as...'), self._main,
            gtk.FILE_CHOOSER_ACTION_SAVE, (gtk.STOCK_CANCEL,
            gtk.RESPONSE_CANCEL, gtk.STOCK_SAVE, gtk.RESPONSE_OK))
        dialog.set_default_response(gtk.RESPONSE_OK)
        self._set_file_filters(dialog)
        
        response = dialog.run()
        if response == gtk.RESPONSE_OK:
            f = open(dialog.get_filename(), 'w')
            if f != None:
                f.write(self._buffer.get_text(self._buffer.get_start_iter(),
                    self._buffer.get_end_iter()) + '\n')
                f.close()
                self._recentmanager.add_item(
                    self._ensure_uri(dialog.get_filename()))
            else:
                msg = gtk.MessageDialog(self._main, gtk.DIALOG_MODAL,
                    gtk.MESSAGE_ERROR, gtk.BUTTONS_OK,
                    _('Could not open file "%s" for saving.') % dialog.get_filename())
                msg.run()
                msg.destroy()
        
        dialog.destroy()
    
    def _e_undo(self, *discard):
        if self._buffer.can_undo():
            self._buffer.undo()
    
    def _e_redo(self, *discard):
        if self._buffer.can_redo():
            self._buffer.redo()
    
    def _e_cut(self, *discard):
        if self._inputbox.get_editable():
            self._buffer.cut_clipboard(gtk.Clipboard(), True)
        else:
            self._e_copy()

    def _e_copy(self, *discard):
        self._buffer.copy_clipboard(gtk.Clipboard())

    def _e_paste(self, *discard):
        if self._inputbox.get_editable():
            self._buffer.paste_clipboard(gtk.Clipboard(), None, True)
    
    def _e_delete(self, *discard):
        if self._inputbox.get_editable():
            self._buffer.delete_selection(True,True)

    def _e_voice_gender_toggled(self, menuitem, gender):
        if menuitem.get_active():
            settings.set_string(settings_gender, gender)

    def _e_punctuation_toggled(self, menuitem, punctuation):
        if menuitem.get_active():
            settings.set_string(settings_punctuation, punctuation)
        espeak.set_parameter(espeak.Parameter.Punctuation,
             getattr(espeak.Punctuation, punctuation.capitalize()))
    
    #def _e_on_context_menu(self, entry, menu):
    #
    #    # Add a entry to the contextual menu
    #    menulabel = gtk.MenuItem(_("Language"))
    #    menu.append(menulabel)
    #    
    #    # Populate it with all available languages
    #    languages = gtk.Menu()
    #    menulabel.set_submenu(languages)
    #    text = self._buffer.get_text(self._buffer.get_start_iter(),
    #        self._buffer.get_end_iter())
    #    priority = language.identify(text)
    #    automatic = gtk.RadioMenuItem(label=_("Detect automatically"))
    #    languages.append(automatic)
    #    position = 1
    #    languages.append(gtk.SeparatorMenuItem())
    #    for voice in self._get_voice_list():
    #        item = gtk.RadioMenuItem(group=automatic, label=voice.name.capitalize())
    #        if voice.name.lower() in priority:
    #            languages.insert(item, position)
    #            position += 1
    #        else:
    #            languages.append(item)
    #    # Make all new entries visible
    #    menu.show_all()
    
    def _e_quit(self, *discard):
        espeak.set_SynthCallback(None)
        gtk.main_quit()
    
    def _e_about(self, *discard):
        self._about_dialog.run()

def main():
        
    app = Application()



# -*- coding: utf-8 -*-
#
# Graphical interface for the eSpeak speech synthesizer
#
# Copyright © 2009-2010 Siegfried-Angel Gevatter Pujals <siegfried@gevatter.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

import os
import sys
import gconf
import locale
import gettext
import logging
import optparse
import subprocess

# Constants

version = '0.3'
basedir = os.path.join(os.path.dirname(__file__), '..')
# It's really a .ui file, but distutils i18n wants .glade
interface_file = os.path.join(basedir, 'data/interface.glade')
icon_file = os.path.join(basedir, 'data/espeak-gui.png')

# Internationalization

gettext.install('espeak-gui', unicode=True)
locale.setlocale(locale.LC_ALL, "") # for unicode sorting with locale.strcoll

# GConf

settings = gconf.Client()

settings_prefix = '/apps/espeak-gui'
settings_selected_voice = '%s/voice/language' % settings_prefix
settings_speed = '%s/speed' % settings_prefix
settings_punctuation = '%s/punctuation' % settings_prefix
settings_gender = '%s/voice/gender' % settings_prefix
settings.add_dir('/apps/espeak-gui', gconf.CLIENT_PRELOAD_RECURSIVE)

# OptionParser

parser = optparse.OptionParser(version=version)
options, arguments = parser.parse_args()

# Start another instance of espeak-gui
def new_instance():
    subprocess.Popen([sys.argv[0]])

__all__ = [
    'version',
    'interface_file', 'icon_file',
    'settings', 'settings_selected_voice', 'settings_speed',
    'settings_punctuation', 'settings_gender',
    'options', 'arguments', 'new_instance'
]



# -*- coding: utf-8 -*-
#
# Graphical interface for the eSpeak speech synthesizer
#
# Copyright © 2009-2012 Siegfried-Angel Gevatter Pujals <siegfried@gevatter.com>
# Copyright © 2009 Joe Burmeister <joe.a.burmeister@googlemail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

import gtk

from resources import version

_gnulicense = \
"This program is free software: you can redistribute it and/or modify\n\
it under the terms of the GNU General Public License as published by\n\
the Free Software Foundation, either version 3 of the License, or\n\
(at your option) any later version.\n\
\n\
This program is distributed in the hope that it will be useful,\n\
but WITHOUT ANY WARRANTY; without even the implied warranty of\n\
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the\n\
GNU General Public License for more details.\n\
\n\
You should have received a copy of the GNU General Public License\n\
along with this program.  If not, see <http://www.gnu.org/licenses/>."

def create_about_dialog(parent_window):
    dialog = gtk.AboutDialog()
    dialog.set_version(version)
    dialog.set_program_name('espeak-gui')
    dialog.set_icon(parent_window.get_icon())
    dialog.set_license(_gnulicense)
    dialog.set_copyright(
        'Copyright © 2009-2010 Siegfried-Angel Gevatter Pujals\n' \
        'Copyright © 2009 Joe Burmeister')
    dialog.set_authors([
        'Siegfried-Angel Gevatter Pujals <siegfried@gevatter.com>',
        'Joe Burmeister <joe.a.burmeister@googlemail.com>'])
    dialog.set_logo(dialog.get_icon())
    
    def close(w, res=None):
        w.hide()
    
    dialog.connect('response', close)
    dialog.connect('destroy', close)
    
    return dialog



# -*- coding: utf-8 -*-
#
# Graphical interface for the eSpeak speech synthesizer
#
# Copyright © 2010-2012 Siegfried-Angel Gevatter Pujals <siegfried@gevatter.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

import os
import locale
import ctypes
from xdg import BaseDirectory

class LanguageIdentifier:

    LIBTEXTCAT_DIR = '/usr/share/libtextcat/'

    _libtextcat = None
    _handle = None

    def __init__(self):
        self._conf_file = self._get_list_filename()
        self._libtextcat = ctypes.CDLL('libtextcat.so.0', ctypes.RTLD_GLOBAL)
        self._libtextcat.textcat_Init.restype = ctypes.c_void_p
        self._libtextcat.textcat_Classify.restype = ctypes.c_char_p
        self._libtextcat.textcat_Version.restype = ctypes.c_char_p
        self._handle = self._libtextcat.textcat_Init(self._conf_file)
        self._languages = self._get_language_list(self._conf_file)

    def __del__(self):
        self._libtextcat.textcat_Done(self._handle)

    def identify(self, text):
        result = self._libtextcat.textcat_Classify(self._handle, text, len(text))
        if result in ('SHORT', 'UNKNOWN'):
            return []
        return result.replace('[', '').split(']')[:-1]

    @classmethod
    def _get_list_filename(cls):
        #conffile = os.path.join(cls.LIBTEXTCAT_DIR, 'conf.txt')
        #if os.path.isfile(conffile):
        #    return conffile
        return cls._generate_list(os.path.join(cls.LIBTEXTCAT_DIR, 'LM'))

    @staticmethod
    def _generate_list(path):
        tmpfilename = os.path.join(BaseDirectory.xdg_cache_home,
            'espeak-gui/libtextcat_conf.txt')
        try:
            os.makedirs(os.path.dirname(tmpfilename))
        except OSError:
            pass
        with open(tmpfilename, 'w') as tempfile:
            for filename in (os.path.join(path, name) for name in os.listdir(path)):
                print >>tempfile, "%s\t%s" % (filename,
                    filename.split('/')[-1].split('.', 1)[0])
        return tmpfilename
    
    @staticmethod
    def _get_language_list(filename):
        with open(filename, 'r') as conffile:
            lines = conffile.read().split('\n')
            lines = map(lambda x: x.split('#')[0].strip(), lines)
            lines = [''.join(line.split('\t')[1:]) for line in lines]
            return filter(None, lines)
        return None
    
    @property
    def languages(self):
        return self._languages

class LanguageManager:

    _identifier = None

    _languages_autodetect = None
    _languages_other = None

    _lang2identifier = None         # for espeak-only stuff
    _lang2iso = None                # for libtextcat -> espeak conversion
    _supported_languages = None
    _last_language = None

    def __init__(self, espeak_voices):
        try:
            self._identifier = LanguageIdentifier()
        except Exception, e:
            print "Couldn't initialize language identifier: %s" % e
        
        self._languages_autodetect = []
        self._languages_other = []
        self._lang2identifier = {}
        self._lang2iso = {}
        self._supported_languages = []
        
        for voice in espeak_voices:
            voice_name = voice.name.lower()
            self._register_language(voice_name, voice.identifier)
            if self._identifier and voice_name in self._identifier.languages:
                self._languages_autodetect.append(voice)
                self._supported_languages.append(voice_name)
            else:
                self._languages_other.append(voice)

    def _register_language(self, name, identifier):
        self._lang2identifier[name] = identifier
        iso_name = name
        if name.endswith('-test'):
            iso_name = name[:-5]
        self._lang2iso[iso_name] = identifier.split('/', 1)[-1]

    def get_languages(self, autodetect=None):
        result = []
        if autodetect or autodetect is None:
            result.extend(self._languages_autodetect)
        if not autodetect or autodetect is None:
            result.extend(self._languages_other)
        return result

    def get_default_language(self):
        try:
            language = locale.getlocale()[0].split('_')[0]
        except Exception:
            pass
        
        if language in self._lang2identifier.values():
            return language
        return 'en'

    def autodetect(self, text):
        if not self._identifier:
            # not supported
            return LanguageGuess([], self.get_default_language())
        candidates = self._identifier.identify(text)
        candidates = [x for x in candidates if x in self._supported_languages]
        return LanguageGuess(candidates, self._choose_language(candidates))

    @property
    def autodetect_supported(self):
        return self._identifier

    def _get_iso_from_lang(self, name):
        try:
            return self._lang2iso[name.lower()]
        except KeyError:
            print 'Can\'t translate "%s" to ISO code.' % name
            return None

    def _choose_language(self, candidates):
        language = None
        if candidates:
            # We use to first guessed candidate for which we have an identifier
            langs = list(candidates)
            langs.reverse()
            while langs and not language:
                language = self._get_iso_from_lang(langs.pop())
        elif self._last_language:
            # If that fails, we use the previous guessed language 
            language = self._last_language
        else:
            # If that fails too, we use the default language
            language = self.get_default_language()
        
        self._last_language = language
        return language

class LanguageGuess:

    _candidates = None
    _language = None

    def __init__(self, candidates, language):
        self._candidates = candidates
        self._language = language

    @property
    def good(self):
        return self._candidates

    @staticmethod
    def _cut_string(string, length):
        if len(string) <= length:
            return string
        return "%s..." % string[:length-3]

    def get_display_name(self):
        label = ', '.join(x.capitalize() for x in self._candidates)
        return self._cut_string(label, 25)

    def get_language(self):
        return self._language

if __name__ == '__main__':
    l = LanguageIdentifier()
    # English
    print l.identify("Over the coming weeks, we will be making two ' \
        'important updates that will impact how you interact...")
    # Catalan
    print l.identify('Hola, aquí tens la nova versió del programa. Caldria ' \
        'afegir un nou paràgraf al final de la descripció. ' \
        'Qualsevol cosa em dius. Moltes gràcies!')



# -*- coding: utf-8 -*-
#
# Graphical interface for the eSpeak speech synthesizer
#
# Copyright © 2009-2012 Siegfried-Angel Gevatter Pujals <siegfried@gevatter.com>
# Copyright © 2009 Joe Burmeister <joe.a.burmeister@googlemail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

import gtk
import gobject

class VoiceBox:

    AUTODETECT_LABEL = _("Detect automatically")
    SEPARATOR_KEY = 'separator'

    _gui_object = None
    _language_manager = None

    def __init__(self, gui_object, language_manager):
        self._gui_object = gui_object
        self._language_manager = language_manager
        self._initialize()

    def _initialize(self):
        store = gtk.ListStore(gobject.TYPE_STRING, gobject.TYPE_STRING)
        self._gui_object.set_model(store)
        
        languages_autodetect = self._language_manager.get_languages(autodetect=True)
        languages_other = self._language_manager.get_languages(autodetect=False)
        
        if self._language_manager.autodetect_supported:
	        store.append([self.AUTODETECT_LABEL, 'autodetect'])
        if languages_autodetect:
	        store.append(['Available Languages', self.SEPARATOR_KEY])
        for voice in languages_autodetect:
            store.append([voice.name.capitalize(), voice.identifier])
        if languages_other and self._language_manager.autodetect_supported:
	        store.append(['Languages without auto-detection', self.SEPARATOR_KEY])
        for voice in languages_other:
            store.append([voice.name.capitalize(), voice.identifier])
    
        cellr = gtk.CellRendererText()
        self._gui_object.pack_start(cellr)
        self._gui_object.add_attribute(cellr, 'text', 0)
        self._gui_object.set_row_separator_func(lambda model, _iter:
            model.get_value(_iter, 1) == self.SEPARATOR_KEY)
        self._gui_object.connect('key-press-event', self._on_voicebox_keypress)

    def get_model(self):
        return self._gui_object.get_model()

    def set_voice(self, voice):
        values = [x[1] for x in self._gui_object.get_model()]
        try:
            self._gui_object.set_active(values.index(voice))
        except ValueError:
            print "Voice \"%s\" is not available." % voice
    
    def get_voice(self):
        """
        Returns a tuple (voice_name, voice_id).
        """
        store = self._gui_object.get_model()
        active = store[self._gui_object.get_active()]
        return (active[0], active[1])
    
    def set_guess(self, guess):
        store = self._gui_object.get_model()
        store[0][0] = self.AUTODETECT_LABEL if not guess.good else \
            '%s [%s]' % (self.AUTODETECT_LABEL, guess.get_display_name())
    
    def _on_voicebox_keypress(self, voicebox, event):
        """ Iterates the selection over the languages starting with the
            typed character. """
        position = self._gui_object.get_active()
        store = self._gui_object.get_model()
        letter = event.string.upper()
        if not letter:
            return
        self._gui_object.set_active(
           	 self._find_voice_position(start=letter, from_position=position+1))
    
    def _find_voice_position(self, start=None, complete=None, from_position=0):
        """ Returns the position of the first voice whose name matches
            the value provided in `complete` or starts with the character
            given in `start`.
            
            The optional parameter `from_position' sets the position from
            which _find_voice_position should start searching. If it is
            different from zero, _find_voice_position will start searching
            again from the start, until completing the circle (reaching the
            value in from_position).
            
            Only one of `start` and `complete` can be used at the same time. """
        
        store = self._gui_object.get_model()
        needle = start or complete
        values = [x[0] if x[1] != self.SEPARATOR_KEY else ' ' for x in store]
        if start:
            values = [value[0] for value in values]
        try:
            return values[from_position:].index(needle) + from_position
        except ValueError:
        	try:
        		return values[:from_position].index(needle)
        	except ValueError:
	            return -1



#! /usr/bin/env python

import os
from glob import glob
from distutils.core import setup
from distutils.command.install import install as orig_install
from DistUtilsExtra.command import *

class _install(orig_install):
    
    def _create_symlink(self, src, dst):
        src = os.path.join(self.prefix, src)
        dst = os.path.join(self._destdir, dst)
        if not os.path.islink(dst):
            print 'Creating symlink "%s"...' % dst
            os.symlink(src, dst)
    
    def _create_directory(self, dir):
        dir = os.path.join(self._destdir, dir)
        if not os.path.isdir(dir):
            print 'Creating directory "%s"...' % dir
            os.mkdir(dir)
    
    def run(self):
        if self.root and self.prefix:
            self._destdir = os.path.join(self.root, self.prefix.strip('/'))
        else:
            self._destdir = self.prefix
        orig_install.run(self)
        # Ensure the needed directories exist
        self._create_directory('bin')
        self._create_directory('share/pixmaps')
        # Create a symlink for the executable file
        self._create_symlink('share/espeak-gui/espeak-gui', 'bin/espeak-gui')
        # Create a symlink for the icon
        self._create_symlink('share/espeak-gui/data/espeak-gui.png',
            'share/pixmaps/espeak-gui.png')

setup(
    name = 'espeak-gui',
    version = '0.3',
    description = 'Graphical user interface for the eSpeak speech synthesizer',
    author = 'Siegfried-A. Gevatter Pujals (and contributors)',
    author_email = 'siegfried@gevatter.com',
    url = 'https://launchpad.net/espeak-gui',
    license = 'GPL',
    platforms = 'posix',
    data_files = [
        ('share/espeak-gui', ['espeak-gui']),
        ('share/espeak-gui/data', glob('data/*')),
        ('share/espeak-gui/src', glob('src/*')),
        ],
    cmdclass = {
        'install': _install,
        'build': build_extra.build_extra,
        'build_i18n': build_i18n.build_i18n,
        #'build_help':  build_help.build_help,
        #'build_icons': build_icons.build_icons,
        },
    classifiers = [
        # http://pypi.python.org/pypi?%3Aaction=list_classifiers
        'License :: OSI-Approved :: GNU General Public License (GPL)',
        'Intended Audience :: End Users/Desktop',
        'Development Status :: 3 - Alpha',
        'Topic :: Multimedia :: Sound/Audio :: Speech',
        'Topic :: Multimedia :: Sound/Audio :: Sound Synthesis',
        'Topic :: Adaptive Technologies',
        'Programming Language :: Python',
        'Environment :: X11 Applications :: GTK',
        ]
    )
